// Practice1.cpp : Defines the entry point for the console application.
//
//
// written 9/8/2009 - Mike Gleicher
//
// usage "practice1 infile outfile sat x y"
//
#define NOMINMAX

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <algorithm>
#include "TargaImage.H"

// utility - check to make sure the file name appears to be a tga file
// really simple - just looks for the suffix
int checkTgaSuffix(const char* fname)
{
	if (fname) {
		const char* sf = strrchr(fname,'.');
		if (sf) {
			// case insensitive comparison - returns 0 if matches
			return ! _stricmp(sf,".tga");
		}
	}
	return 0;
}

//entry point
int main(int argc, char* argv[])
{
	if (argc != 6) {
		printf("practice 1 requires 5 arguments: infile outfile sat x y\n");
		return(1);
	}

	// quick check to make sure that the file names are really file names
	// (see if there's a ".")
	if (!checkTgaSuffix(argv[1])) {
		printf("first argument doesn't seem to be a file name\n");
		return(2);
	}
	if (!checkTgaSuffix(argv[2])) {
		printf("second argument doesn't seem to be a file name\n");
		return(3);
	}

	TargaImage* image = TargaImage::readImage(argv[1]);
	if (!image) {
		printf("Can't open input file `%s'\n",argv[1]);
		return(4);
	}

	// if we're supposed to be desaturating, do it
	float desat = static_cast<float>(atof(argv[3]));
	float sat = 1-desat;		// its handy to have

	if(desat>0) {
		// the pixels are all just in order, so just run through them
		unsigned char* imPtr = image->pixels();

		for(unsigned int r=0; r<image->height(); r++) {
			// note that we advance to the next pixel (4 channels, since there's always alpha)
			for(unsigned int c=0; c<image->width(); c++, imPtr+=4) {
				// get the values in float - just to make the math easy
				float fR = static_cast<float>(imPtr[0]);
				float fG = static_cast<float>(imPtr[1]);
				float fB = static_cast<float>(imPtr[2]);
				float l = 0.3f * fR + 0.59f * fG + .11f * fB;

				fR = desat * l + sat * fR;
				fG = desat * l + sat * fG;
				fB = desat * l + sat * fB;

				imPtr[0] = static_cast<unsigned char>(fR);
				imPtr[1] = static_cast<unsigned char>(fG);
				imPtr[2] = static_cast<unsigned char>(fB);
			}
		}
	}

	// now to make the box - make sure we don't go out of bounds!
	unsigned int x = atoi(argv[4]);
	unsigned int y = atoi(argv[5]);

	const unsigned int boxsize=20;

	// upper bound on x and y - edge of box and image
	unsigned int yb = std::min(y+boxsize+1, image->height());
	unsigned int xb = std::min(x+boxsize+1, image->width());

	// lower bound - can't just do subtraction since unsigned
	for(unsigned int r = ((y>boxsize) ? y-boxsize : 0) ; r<yb; r++)
		for(unsigned int c= ((x>boxsize) ? x-boxsize : 0) ; c<xb; c++) {
			unsigned char* ptr = image->pixels() + (r * image->width() * 4) + (c*4);
			ptr[0] = 0;
			ptr[1] = 255; // green
			ptr[2] = 0;
			ptr[3] = 255;
		}

	// we're done, write it back out
	image->write(argv[2]);

	

	return 0;
}

